<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive"
	xmlns:of="scpf.org:office" xmlns:mobile="http://scenari.soreha.fr/sc-mobile"
	exclude-result-prefixes="mobile sc sp of">
	
	<xsl:import href="bsco:rss.xsl" />
	
	<xsl:output omit-xml-declaration="yes" indent="no" method="xml" />

	<xsl:variable name="vBaseUrl">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">baseUrl</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:template match="of:sTxt">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="of:sTxt/text()" />
	
	<xsl:template match="sc:para">
		<p>
			<xsl:apply-templates />
		</p>
	</xsl:template>
	
	<xsl:template match="sc:itemizedList">
		<li>
			<xsl:apply-templates />
		</li>
	</xsl:template>
	
	<xsl:template match="sc:orderedList">
		<ol>
			<xsl:apply-templates />
		</ol>
	</xsl:template>
	
	<xsl:template match="sc:listItem">
		<li>
			<xsl:apply-templates />
		</li>
	</xsl:template>
	
	<xsl:template match="sc:uLink[@role='url']">
		<a href="{@url}">
			<span>
				<xsl:apply-templates />
			</span>
		</a>
	</xsl:template>
	
	<xsl:template match="sc:phrase[@role='quote']">
		<q>
			<span>«</span>
			<xsl:apply-templates />
			<span>»</span>
		</q>
	</xsl:template>
	
	<xsl:template match="sc:phrase[@role='special']">
		<em>
			<xsl:apply-templates />
		</em>
	</xsl:template>
	
	<xsl:template match="sc:inlineStyle[@role='emphasis']">
		<strong>
			<xsl:apply-templates />
		</strong>
	</xsl:template>
	
	<xsl:template match="sc:inlineImg[@role='ico']">
		<xsl:variable name="vImageContent" select="getContent(gotoSubModel(), 'small')" />
		<xsl:apply-templates select="parseXml($vImageContent)/img" />
	</xsl:template>
	
	<xsl:template match="text()">
		<xsl:copy-of select="." />
	</xsl:template>
	
	<xsl:template match="img">
		<xsl:copy>
			<xsl:copy-of select="@width | @height" />
			<xsl:attribute name="src">
				<xsl:value-of select="concat($vBaseUrl, '/', @src)"/>
			</xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
</xsl:stylesheet>
