<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" xmlns="scpf.org:webMedia/sorehaPhone" exclude-result-prefixes="sc sp of wm java" xmlns:java="http://xml.apache.org/xslt/java">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	<xsl:param name="vArguments"/>
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<!-- Récupère un paramètre passé par lors de l'appel de l'agent -->
	<xsl:template name="tGetParam">
		<xsl:param name="pName"/>
		<xsl:if test="string-length(normalize-space($pName))&gt;0 and string-length(normalize-space($vArguments))&gt;0">
			<xsl:value-of select="substring-before(substring-after($vArguments, concat($pName,'=')), ';')"/>
		</xsl:if>
	</xsl:template>
	<xsl:variable name="vStart">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">start</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	<xsl:template match="wm:segment">
		<xsl:variable name="vClipBegin" select="wm:segmentM//sp:begin"/>
		<xsl:variable name="vClipEnd" select="wm:segmentM//sp:end"/>
		<xsl:variable name="vDuration" select="$vClipEnd - $vClipBegin"/>
		<xsl:variable name="vEnd" select="$vStart + $vDuration"/>
		<xsl:variable name="vMediaType" select="normalize-space(computeStrAgent(concat(concat('@', switchAxis(codeAgent(lookForAscendants($vDialog,'currentToRoot','wm_segmentation')), 'mediaType')), '/xhtmlContent')))"/>
		<xsl:variable name="vMediaUri" select="normalize-space(computeStrAgent(concat(concat('@', switchAxis(codeAgent(lookForAscendants($vDialog,'currentToRoot','wm_segmentation')), if($vMediaType='video', 'mediaUriMp4', 'mediaUriMp3'))), '/xhtmlContent')))"/>
		<xsl:variable name="vTitle" select="getFullTitleText(currentModel(), 'index')"/>
		<xsl:variable name="vMediaErrorTitle" select="'Erreur : le média ou une de ses alternatives doit être au format MP3 ou MP4.'"/>
		<segment title="{if($vMediaUri, $vTitle, $vMediaErrorTitle)}" mediaUri="{$vMediaUri}" mediaType="{$vMediaType}" start="{floor($vStart * 10) div 10}" end="{floor($vEnd * 10) div 10}" clipBegin="{floor($vClipBegin * 10) div 10}" clipEnd="{floor($vClipEnd * 10) div 10}">
			<xsl:for-each select="sp:timeRange">
				<xsl:apply-templates select=".">
					<xsl:with-param name="pStart" select="$vStart + .//wm:timeCode//sp:begin"/>
				</xsl:apply-templates>
			</xsl:for-each>
			<xsl:apply-templates select="sp:marginNotes"/>
			<xsl:apply-templates select="sp:complement"/>
		</segment>
	</xsl:template>
	<xsl:template match="sp:timeRange">
		<xsl:param name="pStart"/>
		<xsl:value-of select="computeStrAgent(concat('@', gotoSubModel(), '_Aindex/xhtmlContent'),concat('start=', $pStart, ';'))" disable-output-escaping="yes"/>
	</xsl:template>
	<xsl:template match="sp:marginNotes">
		<xsl:apply-templates select="./wm:illustrationText//sc:uLink"/>
	</xsl:template>
	<xsl:template match="sc:uLink[@role='glos'] ">
		<reference type="glossary">
			<title>
				<xsl:value-of select="."/>
			</title>
			<desc>				
				<xsl:value-of select="getContent(gotoSubModel())" disable-output-escaping="yes"/>		
			</desc>
		</reference>
	</xsl:template>
	<xsl:template match="sc:uLink[@role='ref']">
		<reference type="index">
			<title>
				<xsl:value-of select="."/>
			</title>
			<desc>				
				<xsl:value-of select="getContent(gotoSubModel())" disable-output-escaping="yes"/>		
			</desc>
		</reference>
	</xsl:template>
	<xsl:template match="sc:uLink[@role='bib']">
		<reference type="bibliography">
			<title>
				<xsl:value-of select="."/>
			</title>			
			<desc>				
				<xsl:value-of select="getContent(gotoSubModel())" disable-output-escaping="yes"/>		
			</desc>
		</reference>
	</xsl:template>
	<xsl:template match="sp:complement">
		<complement title="{getShortTitleText(gotoSubModel(), 'index')}" refUri="{getUrl(gotoSubModel(), 'nav')}">
			<xsl:value-of select="getContent(gotoSubModel(), 'index')" disable-output-escaping="yes"/>
		</complement>
	</xsl:template>
	<xsl:template match="* | text()"/>
</xsl:stylesheet>