<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" xmlns="scpf.org:webMedia/sorehaPhone" xmlns:java="http://xml.apache.org/xslt/java" xmlns:date="http://xml.apache.org/xalan/java/java.util.Date" xmlns:dateFormat="http://xml.apache.org/xalan/java/java.text.SimpleDateFormat" xmlns:locale="http://xml.apache.org/xalan/java/java.util.Locale" exclude-result-prefixes="sc sp of wm java date dateFormat locale">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	<xsl:param name="vArguments"/>
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<!-- Transforme une date en un format quelconque -->
	<xsl:template name="tFormatDate">
		<xsl:param name="pYear"/>
		<xsl:param name="pMonth"/>
		<xsl:param name="pDay"/>
		<xsl:param name="pHours"/>
		<xsl:param name="pMinutes"/>
		<xsl:param name="pSeconds"/>
		<xsl:param name="pFormat"/>
		<xsl:param name="pLanguage"/>
		<xsl:variable name="vFormatter" select="if($pFormat, if($pLanguage, dateFormat:new($pFormat, locale:new($pLanguage)), dateFormat:new($pFormat)), dateFormat:new())"/>
		<xsl:variable name="vDate" select="date:new()"/>
		<xsl:value-of select="if($pYear, date:setYear($vDate, $pYear - 1900))"/>
		<xsl:value-of select="if($pMonth, date:setMonth($vDate, $pMonth - 1))"/>
		<xsl:value-of select="if($pDay, date:setDate($vDate, $pDay))"/>
		<xsl:value-of select="if($pHours, date:setHours($vDate, $pHours))"/>
		<xsl:value-of select="if($pMinutes, date:setMinutes($vDate, $pMinutes))"/>
		<xsl:value-of select="if($pSeconds, date:setSeconds($vDate, $pSeconds))"/>
		<xsl:value-of select="dateFormat:format($vFormatter, $vDate)"/>
	</xsl:template>
	<xsl:template match="wm:montage">
		<xsl:variable name="vDurationsContent" select="getContent(currentModel(), 'durations')"/>
		<xsl:variable name="vDurations" select="parseXml($vDurationsContent)/durations"/>
		<xsl:variable name="vDuration" select="sum($vDurations/duration)"/>
		<webRadio title="{getFullTitleText(currentModel(), 'index')}" duration="{floor($vDuration * 10) div 10}">
			<xsl:attribute name="modificationDate">
				<xsl:call-template name="tFormatDate">
					<xsl:with-param name="pFormat">yyyy-MM-dd HH:mm</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:for-each select="sp:segmentationFull | sp:segment">
				<xsl:variable name="vPosition" select="position()"/>
				<xsl:apply-templates select=".">
					<xsl:with-param name="pStart" select="sum($vDurations/duration[position() &lt; $vPosition])"/>
				</xsl:apply-templates>
			</xsl:for-each>
			<xsl:apply-templates select="sp:notice"/>
			<xsl:apply-templates select="sp:extras|sp:credits"/>
		</webRadio>
	</xsl:template>
	<xsl:template match="wm:montageM | sp:commonMeta | of:commonM">
		<xsl:apply-templates/>
	</xsl:template>
	<xsl:template match="sp:copyright">
		<xsl:attribute name="author">
			<xsl:value-of select="."/>
		</xsl:attribute>
	</xsl:template>
	<xsl:template match="sp:segmentationFull | sp:segment">
		<xsl:param name="pStart"/>
		<xsl:variable name="vRefId" select="wm:subSegmentRef/wm:subSegmentRefM/sp:refId"/>
		<xsl:variable name="vSubModel" select="if(self::sp:segment, wm:subSegmentRef/wm:subSegmentRefM/sp:refItem, .)"/>
		<xsl:value-of select="computeStrAgent(concat('@', gotoSubModel($vSubModel), '_Aindex/xhtmlContent'),concat('refId=', $vRefId, ';start=', $pStart, ';'))" disable-output-escaping="yes"/>
	</xsl:template>
	<xsl:template match="sp:notice">
		<extra refUri="{getUrl(gotoSubModel(), 'nav')}" title="Infos">
			<xsl:value-of select="getContent(gotoSubModel(),'nav')" disable-output-escaping="yes"/>
		</extra>
	</xsl:template>

	<xsl:template match="sp:extras|sp:credits">
		<extra refUri="{getUrl(gotoSubModel(), 'nav')}" title="{getFullTitleText(gotoSubModel(), 'nav')}">
			<xsl:value-of select="getContent(gotoSubModel(),'index')" disable-output-escaping="yes"/>
		</extra>
	</xsl:template>
	<xsl:template match="* | text()"/>
</xsl:stylesheet>