<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive"
	xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" exclude-result-prefixes="sc sp of wm">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes" />
	<xsl:param name="vDialog" />
	<xsl:param name="vAgent" />
	<xsl:param name="vArguments" />
	<xsl:variable name="vRefId">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">
				refId
			</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	<xsl:template name="tFormatDuration">
		<xsl:param name="pDuration" />
		<xsl:variable name="vHours" select="floor($pDuration div 3600)" />
		<xsl:variable name="vMinutes" select="floor($pDuration mod 3600 div 60)" />
		<xsl:variable name="vSeconds" select="round($pDuration mod 60)" />
		<xsl:if test="$vHours">
			<xsl:value-of select="$vHours" />
			<xsl:text>:</xsl:text>
			<xsl:if test="$vMinutes &lt; 10">
				<xsl:text>0</xsl:text>
			</xsl:if>
		</xsl:if>
		<xsl:value-of select="$vMinutes" />
		<xsl:text>:</xsl:text>
		<xsl:if test="$vSeconds &lt; 10">
			<xsl:text>0</xsl:text>
		</xsl:if>
		<xsl:value-of select="$vSeconds" />
	</xsl:template>
	<xsl:variable name="vOptionsContent" select="getContent(gotoMeta(rootModel()), 'options')" />
	<xsl:variable name="vOptions" select="parseXml($vOptionsContent)/wm:optsPubHtmlM" />
	<xsl:template match="wm:montage">
		<xsl:variable name="vDurationsContent" select="getContent(currentModel(), 'durations')" />
		<xsl:variable name="vDuration" select="sum(parseXml($vDurationsContent)/durations/duration)" />
		<xsl:variable name="vTimelineToc" select="not($vOptions/sp:tocPos) or $vOptions/sp:tocPos = 'timeline'" />
		<xsl:variable name="vTimelineTags" select="$vOptions/sp:tagsPos = 'timeline'" />
		<div id="controlBar">
			<xsl:variable name="vPreviousTitle">
				<xsl:choose>
					<xsl:when test="$vOptions/sp:navLevel = 'timeRange'">
						<xsl:text>Illustration précédente</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>Segment précédent</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="vNextTitle">
				<xsl:choose>
					<xsl:when test="$vOptions/sp:navLevel = 'timeRange'">
						<xsl:text>Illustration suivante</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>Segment suivant</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

			<button id="previousBtn" title="{$vPreviousTitle}">
				<span>
					<xsl:value-of select="$vPreviousTitle" />
				</span>
			</button>
			<button id="playPauseBtn" title="Pause">
				<span>Lecture / Pause</span>
			</button>
			<button id="nextBtn" title="{$vNextTitle}">
				<span>
					<xsl:value-of select="$vNextTitle" />
				</span>
			</button>
			<xsl:if test="$vTimelineToc">
				<button id="tocDisplay" title="Afficher la table des matières">
					<span>Afficher la table des matières</span>
				</button>
			</xsl:if>
			<xsl:if test="$vTimelineTags">
				<button id="tagsDisplay" title="Afficher les mots clés">
					<span>Afficher les mots clés</span>
				</button>
			</xsl:if>
			<div id="timelineBox">
				<div id="timeline">
					<input value="0" id="seekBtn" title="Barre de navigation" type="range" role="slider" aria-valuemin="0" aria-valuenow="0" />
					<ul class="timeSegments">
						<xsl:apply-templates select="sp:segmentationFull | sp:segment">
							<xsl:with-param name="pDuration" select="$vDuration" />
						</xsl:apply-templates>
					</ul>
				</div>
			</div>
			<div id="timeLabels">
				<span id="currentTimeLabel" title="Position temporelle">0:00</span>
				<span id="totalTimeLabel" title="Temps total">
					<xsl:call-template name="tFormatDuration">
						<xsl:with-param name="pDuration" select="$vDuration" />
					</xsl:call-template>
				</span>
			</div>
			<button id="muteBtn" title="Désactiver le son">
				<span>Désactiver le son</span>
			</button>
			<div class="volumeBk">
				<div id="volumeValue" />
				<input id="volumeBtn" title="Barre de volume" type="range" step="0.1" max="1" role="slider" aria-valuemin="0" aria-valuemax="100" aria-valuenow="100" aria-valuetext="100%" />
			</div>
		</div>
		<xsl:if test="$vTimelineToc">
			<div id="timelineToc" role="dialog">
				<button class="tocTagsClose" title="Fermer la fenêtre">
					<span>Fermer la fenêtre</span>
				</button>
				<xsl:value-of select="getContent(currentModel(), 'toc')" disable-output-escaping="yes" />
				<button class="focusOut">
					<span>Sortir de la fenêtre</span>
				</button>
				<button class="focusOnCloseBtn">
					<span>Retour en début de fenêtre</span>
				</button>
			</div>
		</xsl:if>
		<xsl:if test="$vTimelineTags">
			<div id="timelineTags" role="dialog">
				<button class="tocTagsClose" title="Fermer la fenêtre">
					<span>Fermer la fenêtre</span>
				</button>
				<xsl:value-of select="getContent(currentModel(), 'tags')" disable-output-escaping="yes" />
				<button class="focusOut">
					<span>Sortir de la fenêtre</span>
				</button>
				<button class="focusOnCloseBtn">
					<span>Retour en début de fenêtre</span>
				</button>
			</div>
		</xsl:if>
		<xsl:apply-templates select="sp:segmentationFull | sp:segment" mode="transcript" />
	</xsl:template>
	
	<xsl:template match="sp:segmentationFull | sp:segment" mode="transcript">
		<xsl:value-of select="getContent(gotoSubModel(),'transcript')"  disable-output-escaping="yes" />
	</xsl:template>
	
	<xsl:template match="sp:segmentationFull | sp:segment">
		<xsl:param name="pDuration" />
		<xsl:value-of select="execute(setDialogVar('montagePos', position()))" />
		<xsl:variable name="vRefId" select="wm:subSegmentRef/wm:subSegmentRefM/sp:refId" />
		<xsl:variable name="vSubModel" select="if(self::sp:segment, wm:subSegmentRef/wm:subSegmentRefM/sp:refItem, .)" />
		<xsl:value-of select="computeStrAgent(concat('@', gotoSubModel($vSubModel), '_Atimeline/xhtmlContent'), concat('refId=', $vRefId, ';duration=', $pDuration, ';'))"
			disable-output-escaping="yes" />
	</xsl:template>
</xsl:stylesheet>