<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" exclude-result-prefixes="sc sp of wm">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="no" omit-xml-declaration="yes"/>
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<xsl:param name="vArguments"/>
	<!-- Récupère un paramètre passé par lors de l'appel de l'agent -->
	<xsl:template name="tGetParam">
		<xsl:param name="pName"/>
		<xsl:if test="string-length(normalize-space($pName))&gt;0 and string-length(normalize-space($vArguments))&gt;0">
			<xsl:value-of select="substring-before(substring-after($vArguments, concat($pName,'=')), ';')"/>
		</xsl:if>
	</xsl:template>
	<xsl:variable name="vRefId">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">refId</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="vStandalone">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">standalone</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="vOptionsContent" select="getContent(gotoMeta(rootModel()), 'options')"/>
	<xsl:variable name="vOptions" select="parseXml($vOptionsContent)/wm:optsPubHtmlM"/>
	<xsl:template match="wm:segmentation">
		<xsl:variable name="vMediaContent" select="getContent(gotoMeta(currentModel()), 'media')"/>
		<xsl:variable name="vMedia" select="parseXml($vMediaContent)/*"/>
		<xsl:choose>
			<xsl:when test="$vRefId">
				<xsl:variable name="vSegment" select=".//sp:segment[wm:segment/wm:segmentM/sp:id=$vRefId]"/>
				<xsl:apply-templates select="$vMedia">
					<xsl:with-param name="pStart" select="$vSegment//wm:segmentM//sp:begin"/>
					<xsl:with-param name="pStop" select="$vSegment//wm:segmentM//sp:end"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="$vMedia"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="video | audio">
		<xsl:param name="pStart"/>
		<xsl:param name="pStop"/>
		<xsl:copy>
			<xsl:if test="$pStart">
				<xsl:attribute name="data-clipBegin">
					<xsl:value-of select="$pStart"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$pStop">
				<xsl:attribute name="data-clipEnd">
					<xsl:value-of select="$pStop"/>
				</xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="$vStandalone='true'">
					<xsl:attribute name="id">media</xsl:attribute>
					<xsl:attribute name="preload">auto</xsl:attribute>
					<!-- <xsl:choose>
						<xsl:when test="$vOptions/sp:autoplay = 'no' "/>
						<xsl:otherwise>
							<xsl:attribute name="data-autoplay">true</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose> -->
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="preload">none</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:apply-templates select="@* | addAttribute"/>
			<xsl:apply-templates select="*[not(self::addAttribute)] | text()"/>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="node() | @*">
		<xsl:copy>
			<xsl:apply-templates select="node() | @*"/>
		</xsl:copy>
	</xsl:template>
</xsl:stylesheet>