<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" exclude-result-prefixes="sc sp of wm">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<xsl:param name="vArguments"/>
	<xsl:variable name="vRefId">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">refId</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	<xsl:template name="tFormatDuration">
		<xsl:param name="pDuration"/>
		<xsl:variable name="vHours" select="floor($pDuration div 3600)"/>
		<xsl:variable name="vMinutes" select="floor($pDuration mod 3600 div 60)"/>
		<xsl:variable name="vSeconds" select="round($pDuration mod 60)"/>
		<xsl:if test="$vHours">
			<xsl:value-of select="$vHours"/>
			<xsl:text>:</xsl:text>
		</xsl:if>
		<xsl:if test="$vHours and $vMinutes &lt; 10">0</xsl:if>
		<xsl:value-of select="$vMinutes"/>
		<xsl:text>:</xsl:text>
		<xsl:if test="$vSeconds &lt; 10">0</xsl:if>
		<xsl:value-of select="$vSeconds"/>
	</xsl:template>
	<xsl:variable name="vOptionsContent" select="getContent(gotoMeta(rootModel()), 'options')"/>
	<xsl:variable name="vOptions" select="parseXml($vOptionsContent)/wm:optsPubHtmlM"/>
	<xsl:template match="wm:montage">
		<xsl:variable name="vDurationsContent" select="getContent(currentModel(), 'durations')"/>
		<xsl:variable name="vDuration" select="sum(parseXml($vDurationsContent)/durations/duration)"/>
		<xsl:variable name="vTimelineToc" select="not($vOptions/sp:tocPos) or $vOptions/sp:tocPos = 'timeline'"/>
		<xsl:variable name="vTimelineTags" select="$vOptions/sp:tagsPos = 'timeline'"/>
		<xsl:variable name="vTimelineStopAtSegmentEnd" select="$vOptions/sp:stopAtSegmentEndBtnPos = 'timeline'"/>
		<xsl:if test="$vTimelineToc">
			<div id="timelineToc">
				<button class="tocTagsClose" onclick="document.getElementById('timelineToc').className=''">
					<span>fermer</span>
				</button>
				<xsl:value-of select="getContent(currentModel(), 'toc')" disable-output-escaping="yes"/>
			</div>
		</xsl:if>
		<xsl:if test="$vTimelineTags">
			<div id="timelineTags">
				<button class="tocTagsClose" onclick="document.getElementById('timelineTags').className=''">
					<span>fermer</span>
				</button>
				<xsl:value-of select="getContent(currentModel(), 'tags')" disable-output-escaping="yes"/>
			</div>
		</xsl:if>
		<div id="controlBar">
			<div id="playerBtns">
				<button id="previousBtn" title="Segment précédent">
					<span>|«</span>
				</button>
				<button id="playPauseBtn" title="Lecture / Pause">
					<span>▶||</span>
				</button>
				<button id="nextBtn" title="Segment suivant">
					<span>»|</span>
				</button>
			</div>
			<div id="timelineBox">
				<div id="timeline">
					<div class="timeSegments">
						<xsl:apply-templates select="sp:segmentationFull | sp:segment">
							<xsl:with-param name="pDuration" select="$vDuration"/>
						</xsl:apply-templates>
					</div>
					<div id="timeCursor"/>
				</div>
			</div>
			<div id="playerTools">
				<div id="timeLabels">
					<span id="currentTimeLabel" title="Position temporelle">0:00</span>
					<span id="totalTimeLabel" title="Temps total">
						<xsl:call-template name="tFormatDuration">
							<xsl:with-param name="pDuration" select="$vDuration"/>
						</xsl:call-template>
					</span>
				</div>
				<xsl:if test="$vTimelineToc">
					<button id="tocDisplay" title="Afficher la table des matières"/>
				</xsl:if>
				<xsl:if test="$vTimelineTags">
					<button id="tagsDisplay" title="Afficher les mots clés"/>
				</xsl:if>
				<xsl:if test="$vTimelineStopAtSegmentEnd">
					<button id="stopAtSegmentEndBtn" title="Mettre en pause à la fin des segments" data-checked="true">
						<span>Mettre en pause à la fin des segments</span>
					</button>
				</xsl:if>
			</div>
		</div>
	</xsl:template>
	<xsl:template match="sp:segmentationFull | sp:segment">
		<xsl:param name="pDuration"/>
		<xsl:value-of select="execute(setDialogVar('montagePos', position()))"/>
		<xsl:variable name="vRefId" select="wm:subSegmentRef/wm:subSegmentRefM/sp:refId"/>
		<xsl:variable name="vSubModel" select="si(self::sp:segment, wm:subSegmentRef/wm:subSegmentRefM/sp:refItem, .)"/>
		<xsl:value-of select="resultatAgent(concat('@', gotoSubModel($vSubModel), '_Atimeline/xhtmlContent'), concat('refId=', $vRefId, ';duration=', $pDuration, ';'))" disable-output-escaping="yes"/>
	</xsl:template>
</xsl:stylesheet>