<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" exclude-result-prefixes="sc sp of wm">
	
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<xsl:param name="vArguments"/>
	
	<!-- Récupère un paramètre passé par lors de l'appel de l'agent -->
	<xsl:template name="tGetParam">
		<xsl:param name="pName"/>
		<xsl:if test="string-length(normalize-space($pName))&gt;0 and string-length(normalize-space($vArguments))&gt;0">
			<xsl:value-of select="substring-before(substring-after($vArguments, concat($pName,'=')), ';')"/>
		</xsl:if>
	</xsl:template>
	
	<xsl:variable name="vRefId">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">refId</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="vStart">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">start</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:template match="wm:segmentation">
		<xsl:choose>
			<xsl:when test="$vRefId">
				<xsl:for-each select=".//sp:segment[wm:segment/wm:segmentM/sp:id=$vRefId]">
					<xsl:apply-templates select=".">
						<xsl:with-param name="pStart" select="$vStart" />
					</xsl:apply-templates>
					<xsl:if test="position() != last()">,</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select=".//sp:segment">
					<xsl:apply-templates select=".">
						<xsl:with-param name="pStart" select="$vStart  + .//wm:segmentM//sp:begin" />
					</xsl:apply-templates>
					<xsl:if test="position() != last()">,</xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="sp:segment">
		<xsl:param name="pStart" />
		<xsl:variable name="vDuration" select=".//wm:segmentM//sp:end - .//wm:segmentM//sp:begin"/>
		<xsl:variable name="vEnd" select="$pStart + $vDuration"/>
		<xsl:variable name="vTarget" select="concat('segment_', alphaHash(gotoSubModel()), getDialogVar('montagePos'))"/>
		
		<xsl:text>{"start":</xsl:text>
		<xsl:value-of select="round($pStart * 1000) div 1000"/>
		<xsl:text>,"end":</xsl:text>
		<xsl:value-of select="round($vEnd * 1000) div 1000"/>
		<xsl:text>,"target":"</xsl:text>
		<xsl:value-of select="$vTarget"/>
		
		<xsl:text>","tags":[</xsl:text>
		<xsl:for-each select="wm:segment/wm:segmentM/sp:tagList/sp:tag">
			<xsl:text>"</xsl:text>
			<xsl:value-of select="translate(getFullTitleText(gotoSubModel()), '&quot;', &quot;&apos;&quot;)"/>
			<xsl:text>"</xsl:text>
			<xsl:if test="position() != last()">
				<xsl:text>,</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text>]}</xsl:text>
	</xsl:template>
</xsl:stylesheet>