<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" exclude-result-prefixes="sc sp of wm">
	
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	<xsl:param name="vArguments"/>
	
	<!-- Récupère un paramètre passé par lors de l'appel de l'agent -->
	<xsl:template name="tGetParam">
		<xsl:param name="pName"/>
		<xsl:if test="string-length(normalize-space($pName))&gt;0 and string-length(normalize-space($vArguments))&gt;0">
			<xsl:value-of select="substring-before(substring-after($vArguments, concat($pName,'=')), ';')"/>
		</xsl:if>
	</xsl:template>
	
	<xsl:variable name="vRefId">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">refId</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="vDuration">
		<xsl:call-template name="tGetParam">
			<xsl:with-param name="pName">duration</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>

	<xsl:template match="wm:segmentation">
		<xsl:choose>
			<xsl:when test="$vRefId">
				<xsl:apply-templates select=".//sp:segment[wm:segment/wm:segmentM/sp:id=$vRefId]">
					<xsl:with-param name="pDuration" select="$vDuration" />
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="sp:segment | sp:part">
					<xsl:with-param name="pDuration" select="$vDuration" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="sp:segment">
		<xsl:param name="pDuration" />
		<xsl:variable name="vBegin" select=".//wm:segmentM//sp:begin"/>
		<xsl:variable name="vEnd" select=".//wm:segmentM//sp:end"/>
		<xsl:variable name="vWidth" select="($vEnd - $vBegin) * 100 div $pDuration" /> 
		<xsl:variable name="vTarget" select="concat('segment_', alphaHash(gotoSubModel()), getDialogVar('montagePos'))"/>
		
		<a href="#{$vTarget}" style="width: {$vWidth}%">
			<xsl:attribute name="data-tags">
				<xsl:for-each select="wm:segment/wm:segmentM/sp:tagList/sp:tag">
					<xsl:value-of select="getFullTitleText(gotoSubModel())" />
					<xsl:if test="position() != last()">
						<xsl:text>,</xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:attribute>
			<span>
				<xsl:value-of select="getFullTitleText(gotoSubModel())" />
			</span>
		</a>
	</xsl:template>
		
	<xsl:template match="sp:part">
		<xsl:param name="pDuration" />
		<xsl:variable name="vBegin" select="(.//wm:segmentM//sp:begin)[1]"/>
		<xsl:variable name="vEnd" select="(.//wm:segmentM//sp:end)[last()]"/>
		<xsl:variable name="vDuration" select="$vEnd - $vBegin" />
		<xsl:variable name="vWidth" select="$vDuration * 100 div $pDuration" /> 
		<xsl:variable name="vTarget" select="concat('segment_', alphaHash(gotoSubModel(wm:part/sp:segment[1])), getDialogVar('montagePos'))"/>
		
		<div style="width: {$vWidth}%" class="timePart">
			<a href="#{$vTarget}" style="width: 100%">
				<span>
					<xsl:value-of select="getFullTitleText(gotoSubModel())" />
				</span>
			</a>
			<div class="timePartCo" width="100%">
				<xsl:apply-templates select="wm:part/sp:segment">
					<xsl:with-param name="pDuration" select="$vDuration" />
				</xsl:apply-templates>
			</div>
		</div>
	</xsl:template>
</xsl:stylesheet>