<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive"
	xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" xmlns:itunes="http://www.itunes.com/dtds/podcast-1.0.dtd"
	xmlns:java="http://xml.apache.org/xalan/java"
	xmlns:date="http://xml.apache.org/xalan/java/java.util.Date"
	xmlns:dateFormat="http://xml.apache.org/xalan/java/java.text.SimpleDateFormat"
	xmlns:locale="http://xml.apache.org/xalan/java/java.util.Locale"
	exclude-result-prefixes="sc sp of java date dateFormat locale">

	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" omit-xml-declaration="yes"/>
	
	<xsl:param name="vArguments" />
	<xsl:param name="vDialog"/>
	<xsl:param name="vAgent"/>
	
	<!-- Transforme une date en un format quelconque -->
	<xsl:template name="tFormatDate">
		<xsl:param name="pDate" />
		<xsl:param name="pFormat" />
		<xsl:param name="pLanguage" />

		<xsl:variable name="vInFormatter" select="dateFormat:new('yyyy-MM-dd')" />
		<xsl:variable name="vPos" select="dateFormat:new('yyyy-MM-dd')" />
		<xsl:variable name="vOutFormatter" select="if($pFormat, if($pLanguage, dateFormat:new($pFormat, locale:new($pLanguage)), dateFormat:new($pFormat)), dateFormat:new())" />
		
		<xsl:variable name="vPos" select="java:text.ParsePosition.new(0)" />
		<xsl:variable name="vDate" select="if($pDate, dateFormat:parse($vInFormatter, $pDate, $vPos), date:new())" />
		
		<xsl:value-of select="dateFormat:format($vOutFormatter, $vDate)" />
	</xsl:template>
	
	<xsl:template name="tFormatDateRfc822">
		<xsl:param name="pDate" />
		
		<xsl:call-template name="tFormatDate">
			<xsl:with-param name="pDate" select="$pDate" />
			<xsl:with-param name="pFormat">EEE, d MMM yyyy HH:mm:ss Z</xsl:with-param>
			<xsl:with-param name="pLanguage">en</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="wm:rss">
		<rss version="2.0">
			<channel>
				<xsl:apply-templates />
			</channel>
		</rss>
	</xsl:template>

	<xsl:template match="wm:rssM">
		<xsl:apply-templates />
		<pubDate>
			<xsl:call-template name="tFormatDateRfc822">
				<xsl:with-param name="pDate" select="sp:pubDate" />
			</xsl:call-template>
		</pubDate>
	</xsl:template>

	<xsl:template
		match="sp:title | sp:link | sp:description | sp:language | sp:managingEditor | sp:webMaster | sp:copyright">
		<xsl:element name="{local-name(.)}">
			<xsl:value-of select="." />
		</xsl:element>
	</xsl:template>

	<xsl:template match="sp:webMedia">
		<item>
			<title>
				<xsl:choose>
					<xsl:when test="wm:rssItemM/sp:title">
						<xsl:value-of select="wm:rssItemM/sp:title" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="getFullTitleText(gotoSubModel())" />
					</xsl:otherwise>
				</xsl:choose>
			</title>

			<xsl:variable name="vPubDate">
				<xsl:call-template name="tFormatDateRfc822">
					<xsl:with-param name="pDate" select="wm:rssItemM/sp:pubDate" />
				</xsl:call-template>
			</xsl:variable>
			<pubDate>
				<xsl:value-of select="$vPubDate" />
			</pubDate>

			<xsl:variable name="vGenUrl">
				<xsl:if test="../wm:rssM/sp:param/sp:gen = 'yes'">
					<xsl:value-of select="../wm:rssM/sp:base" />
					<xsl:text>/</xsl:text>
					<xsl:value-of select="getUrl(gotoSubModel(), 'page')" />
				</xsl:if>
			</xsl:variable>
			
			<xsl:variable name="vUrl">
				<xsl:choose>
					<xsl:when test="wm:rssItemM/sp:link">
						<xsl:value-of select="wm:rssItemM/sp:link" />
					</xsl:when>
					<xsl:when test="$vGenUrl">
						<xsl:value-of select="$vGenUrl" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="../wm:rssM/sp:base" />
						<xsl:text>/</xsl:text>
						<xsl:value-of select="substring-before(java:getSrcName(srcFileAgent(concat('@', getIdFromPath(@sc:refUri)))),'.xml')" />
						<xsl:text>/</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

			<guid>
				<xsl:value-of select="$vUrl" />
			</guid>
			<link>
				<xsl:value-of select="$vUrl" />
			</link>

			<xsl:apply-templates select="wm:rssItemM/sp:category" />

			<xsl:value-of
				select="computeStrAgent(concat('@', gotoSubModel(), '_ArssCo/xhtmlContent'),
				concat('url=', $vUrl,';baseUrl=', ../wm:rssM/sp:base,';pubDate=', $vPubDate, ';'))"
				disable-output-escaping="yes" />
		</item>
	</xsl:template>

	<xsl:template match="sp:category">
		<category>
			<xsl:value-of select="." />
		</category>	
	</xsl:template>
	
	<xsl:template match="sp:image">
		<xsl:variable name="vUrl" select="concat(../sp:base, '/', getUrl(gotoSubModel(), 'nav'))" />
		<xsl:variable name="vFullTitle" select="getFullTitleText(gotoSubModel())" />
		<image>
			<title>
				<xsl:choose>
					<xsl:when test="$vFullTitle">
						<xsl:value-of select="$vFullTitle" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="../sp:title" />
					</xsl:otherwise>
				</xsl:choose>
			</title>
			<xsl:apply-templates select="../sp:link" />
			<url>
				<xsl:value-of select="$vUrl" />
			</url>
		</image>
	</xsl:template>

	<xsl:template match="sp:author">
		<itunes:author>
			<xsl:value-of select="." />
		</itunes:author>
	</xsl:template>

	<xsl:template match="* | text()" />

</xsl:stylesheet>
