(function () {

"use strict";

var debug = false;

document.addEventListener('DOMContentLoaded', function () {
	var media = document.getElementById('media'),
		extrasOptions = {
			swfUri: '../js/FlashMediaElement.swf',
			debug: debug
		}
	
	// Initialisation du media principal
	if (media.getAttribute('data-sequencer') == 'true') {
		var childMediaCount = media.children.length;
		mediaExtraSources(media.children, extrasOptions, function(mediaChild) {
			childMediaCount--;
			if (!childMediaCount) {
				var sources = { };
				for (var child = media.firstElementChild; child; child = child.nextElementSibling) {
					if (sources[child.currentSrc]) child = mediaWrapper(sources[child.currentSrc], child);
					else {
						sources[child.currentSrc] = child;
					}
				}
				// Tout les médias fils sont prêts, initialisation du sequenceur
				mediaSequencer(media, { debug: debug });
				webMedia(media);
				Object.keys(sources).forEach(function(source) {
					sources[source].preload = 'auto';
				});
			}
		});
	} else mediaExtraSources(media, extrasOptions, webMedia);
}, false);

function webMedia (media) {
	if (media.hasAttribute("data-autoplay")) {
		if (document.readyState == "complete") media.play();
		else window.addEventListener('load', function () {
			media.play();
		}, false);
	};
	
	// Variables pour les éléments HTML
	var segmentDataElement = document.getElementById('segmentsData'),
		timeRangesDataElement = document.getElementById('timeRangesData'),
		contentDiv = document.getElementById('content'),
		timeController = document.getElementById('controller'),
		currentTimeLabel = document.getElementById('currentTimeLabel'),
		timeCursor = document.getElementById('timeCursor'),
		tocDisplay = document.getElementById('tocDisplay'),
		tagsDisplay = document.getElementById('tagsDisplay'),
		timelineToc = document.getElementById('timelineToc'),
		timelineTags = document.getElementById('timelineTags'),
		nonContinuousPlayBtn = document.getElementById('nonContinuousPlayBtn'),
		stopAtSegmentEndBtn = document.getElementById('stopAtSegmentEndBtn'),
		playPauseBtn = document.getElementById('playPauseBtn'),
		previousBtn = document.getElementById('previousBtn'),
		nextBtn = document.getElementById('nextBtn');
	
	// Variables fonctionnelles
	var segmentsData = JSON.parse(segmentDataElement.text),
		timeRangesData = timeRangesDataElement && JSON.parse(timeRangesDataElement.text),
		lastUpdate = NaN,
		nonContinuousPlay = false,
		stopAtSegmentEnd = false,
		timeUpdateFromHash = false,
		currentTag = '',
		draggingTimeout = null;
	
	// Initialisation des états
	segmentsData.forEach(updateState);
	if (timeRangesData) {
		timeRangesData.forEach(updateState);
		timeRangesData.some(updateContentState);
	}
	updatePlayState();
	
	// Listeners de changement d'états
	media.addEventListener('timeupdate', timeupdate, false);
	[ 'loadedmetadata', 'playing', 'pause', 'ended', 'seeking', 'seeked' ].forEach (function(event) {
		media.addEventListener(event, updatePlayState, false);
	});

	// Changement du currentTime
	function timeupdate(event) {
		if (event && event.target != media) return;
		var currentTime = media.currentTime,
			fromHash = timeUpdateFromHash;
		timeUpdateFromHash = false;
		
		// Mise à jour des états
		segmentsData.forEach(updateState);
		if (timeRangesData) {
			timeRangesData.forEach(updateState);
			timeRangesData.some(updateContentState);
		}
		
		// Mise à jour du temps courant et du curseur de lecture
		currentTimeLabel.textContent = Math.floor(currentTime / 60) + ':' + ("0" + Math.floor(currentTime % 60)).substr(-2);
		if (!timeCursor.hasAttribute('data-dragging')) timeCursor.style.left  = (currentTime * 100  / media.duration) + "%";

		for (var i=0; i<segmentsData.length; i++) {
			var segmentData = segmentsData[i];
			
			if (currentTime >= segmentData.start && currentTime < segmentData.end) {
				if (!fromHash && (lastUpdate < segmentData.start || lastUpdate > segmentData.end) && !timeCursor.hasAttribute('data-dragging')) {
					// Arrivé sur un nouveau segment (ni par changement de hash, ni par drag du curseur de lecture)
					if (stopAtSegmentEnd) {
						media.pause();
						lastUpdate = currentTime;
						return;
					}
					if (nonContinuousPlay && currentTag && segmentData.tags.indexOf(currentTag) == -1) {
						for (var j=i+1; j<segmentsData.length; j++) {
							if (segmentsData[j].tags.indexOf(currentTag) != -1) {
								lastUpdate = segmentsData[j].start;
								media.currentTime = lastUpdate;
								return;
							}
						}
						media.pause();
					}
				}
				
				// Mise à jour du hash
				if (window.location.hash.substr(1) != segmentData.target) {
					window.removeEventListener('hashchange', hashChanged, true);
					window.location.hash = '#' + segmentData.target;
					window.setTimeout(function() {
						window.addEventListener('hashchange', hashChanged, true);
					}, 200)
				}
				
				break;
			}
		}
		lastUpdate = currentTime;
	}
	
	// Gestion du hash
	window.addEventListener('hashchange', hashChanged, true);
	media.addEventListener('loadeddata', function(event) {
		if (event.target == media) hashChanged()
	}, false);
	
	function hashChanged() {
		var target = window.location.hash.substr(1);
		if (target) {
			// Gestion du media fragment description http://foo.com/video.ogg#t=60
			if (target.substr(0,2) == 't=') {
				var timeTarget = parseInt(target.substr(2));
				media.currentTime = timeTarget;
			} else {
				segmentsData.some(function (segmentData) {
					if (segmentData.target == target) {
						timeUpdateFromHash = true;
						media.currentTime = segmentData.start;
						return true;
					}
				});
			}
		}
	}
	
	// Changement d'état
	function updateState (data) {
		var currentTime = media.currentTime;
		var div = document.getElementById(data.target);
		if (!div) return;
		if (currentTime < data.start) div.setAttribute("data-state", "idle");
		else if (currentTime >= data.start && currentTime < data.end) div.setAttribute("data-state", "active");
		else div.setAttribute("data-state", "done");
	}

	// Changement d'état de l'élément content (hasIllustr & notes & imgSolo)
	function updateContentState(data) {
		var currentTime = media.currentTime;
		if (currentTime >= data.start && currentTime < data.end) {
			if (data.hasContent) contentDiv.setAttribute('data-hasIllustr', true);
			else contentDiv.removeAttribute('data-hasIllustr');
			if (data.hasNotes) contentDiv.setAttribute('data-hasNotes', true);
			else contentDiv.removeAttribute('data-hasNotes');
			if (data.hasImgSolo) contentDiv.setAttribute('data-hasImgSolo', true);
			else contentDiv.setAttribute('data-hasImgSolo', false);
			return true;
		}
		return false;
	}
	
	// Changement d'état de lecture
	function updatePlayState (event) {
		if (event && event.target != media) return;
		if (media.seeking) {
			timeController.setAttribute('data-state', 'seeking');
			previousBtn.disabled = true;
			nextBtn.disabled = true;
		} else {
			if (media.paused || media.ended) timeController.setAttribute('data-state', 'paused');
			else timeController.setAttribute('data-state', 'playing');
			previousBtn.disabled = false;
			nextBtn.disabled = false;
		}
	}

	// Drag du curseur de lecture
	timeCursor.addEventListener('mousedown', function(event) {
		if (!timeCursor.hasAttribute('data-dragging')) {
			document.addEventListener('mousemove', timeCursorDragging);
			document.addEventListener('mouseup', timeCursorDragStop);
		}
	}, false);
	
	function timeCursorDragging (event) {
		if (!timeCursor.hasAttribute('data-dragging')) {
			timeCursor.setAttribute('data-dragging', true);
			updatePlayState(event);				
		}
		var parentRect = timeCursor.parentNode.getBoundingClientRect();
		var left = (event.pageX - parentRect.left) * 100 / (parentRect.right - parentRect.left);
		left = Math.min(Math.max(left, 0), 100);
		timeCursor.style.left  = left + "%";
		var newCurrentTime = left * media.duration / 100;
		clearTimeout(draggingTimeout);
		draggingTimeout = setTimeout(function () {
			timeCursor.removeAttribute('data-dragging', true);
			media.currentTime = newCurrentTime;
		}, 100);
	};
	
	function timeCursorDragStop (event) {
		timeCursor.removeAttribute('data-dragging');
		document.removeEventListener('mousemove', timeCursorDragging);
		document.removeEventListener('mouseup', timeCursorDragStop);
	}
	
	// Bouton de lecture
	playPauseBtn.addEventListener('click', function() {
		if (media.paused || media.ended) media.play();
		else media.pause();
	}, false);
	
	// Boutons suivant/précédent
	previousBtn.addEventListener('click', goToPreviousSegment, false);
	nextBtn.addEventListener('click', goToNextSegment, false);
	
	function goToPreviousSegment () {
		var currentTime = media.currentTime;
		for (var i=0; i<segmentsData.length; i++) {
			var segmentData = segmentsData[i];
			if (currentTime >= segmentData.start && currentTime < segmentData.end)  {
				if (currentTime < segmentData.start + 0.5 && i > 0) media.currentTime = segmentsData[i-1].start;
				else media.currentTime = segmentData.start;
				break;
			}
		}
	}
	
	function goToNextSegment () { 
		var currentTime = media.currentTime;
		for (var i=0; i<segmentsData.length; i++) {
			var segmentData = segmentsData[i];
			if (currentTime >= segmentData.start && currentTime < segmentData.end && i<segmentsData.length-1)  {
				if (currentTime > segmentData.end - 0.5 && i > 0) media.currentTime = segmentsData[i+1].end;
				else media.currentTime = segmentData.end;
				break;
			}
		}
	}

	// Listeners clavier
	window.addEventListener('keydown', function(event) {
		switch (event.keyCode) {
		case 32: /* espace */
			if (media.paused()) media.play();
			else media.pause();
			break;
		case 37: /* gauche */
			if (!media.seeking) {
				if (event.shiftKey && !media.seeking) goToPreviousSegment();
				else media.currentTime = media.currentTime - 20;
			}
			break;
		case 39: /* droite */
			if (!media.seeking) {
				if (event.shiftKey) goToNextSegment();
				else media.currentTime = media.currentTime + 20;
			}
			break;
		}
	}, false);
	
	// Gestion des tags
	forEachQuerySelector('.tag', function (tagElement) {
		var tag = tagElement.getAttribute('data-tags');
		tagElement.addEventListener('click', function() {
			currentTag = currentTag == tag ? '' : tag;
			 
			// Un attribut highlighted est posé sur tout les elements possédant un attribut data-tags contenant le tag séléctionné
			forEachQuerySelector('*[data-tags]', function (element) {
				var tags = (element.getAttribute('data-tags') || '').split(',');
				var hasTag = currentTag && tags.some(function(tag) { return tag == currentTag; }); 
				if (hasTag) element.setAttribute('data-highlighted', true);
				else element.removeAttribute('data-highlighted');
			});
		}, false);
	});
	
	// Bouton d'ouverture du sommaire à partir de la timeline
	if (tocDisplay && timelineToc) {
		timelineToc.addEventListener('click', function(event) {
			if(event.target.tagName.toLowerCase() != 'a') event.stopPropagation();
		}, false);
		window.addEventListener('click', function (event) {
			if (event.target == tocDisplay && !timelineToc.className) timelineToc.className = 'active';
			else timelineToc.className = '';
		});
	}

	// Bouton d'ouverture des tags à partir de la timeline
	if (tagsDisplay && timelineTags) {
		timelineTags.addEventListener('click', function(event) {
			if (event.target.tagName.toLowerCase() != 'a') event.stopPropagation(); 
		}, false);
		window.addEventListener('click', function (event) {
			if (event.target == tagsDisplay && !timelineTags.className) timelineTags.className = 'active';
			else timelineTags.className = '';
		});
	}
	
	// Bouton d'activation de la lecture bout à bout
	if (nonContinuousPlayBtn) {
		nonContinuousPlay = nonContinuousPlayBtn.hasAttribute('data-checked');
		nonContinuousPlayBtn.addEventListener('click', function () {
			nonContinuousPlay = !nonContinuousPlay;
			if (nonContinuousPlay) nonContinuousPlayBtn.setAttribute('data-checked', true);
			else nonContinuousPlayBtn.removeAttribute('data-checked');
		}, false);
	}
	
	// Bouton d'activation de la pause en fin de segment
	if (stopAtSegmentEndBtn) {
		stopAtSegmentEnd = stopAtSegmentEndBtn.hasAttribute('data-checked');
		stopAtSegmentEndBtn.addEventListener('click', function () {
			stopAtSegmentEnd = !stopAtSegmentEnd;
			if (stopAtSegmentEnd) stopAtSegmentEndBtn.setAttribute('data-checked', true);
			else stopAtSegmentEndBtn.removeAttribute('data-checked');
		}, false);
	}

	// Gestion des buttons complement on
	forEachQuerySelector('button.compltOn', function (btnElement) {
		var btnTarget = btnElement.getAttribute('data-target');	
		btnElement.addEventListener('click', function() {			
			var divComplt = document.getElementById(btnTarget);			
			divComplt.setAttribute('data-display', true);
		}, false);	
	});

	// Gestion des buttons complement off
	forEachQuerySelector('button.compltOff', function (btnElement) {
		var btnTarget = btnElement.getAttribute('data-target');	
		btnElement.addEventListener('click', function() {			
			var divComplt = document.getElementById(btnTarget);			
			divComplt.setAttribute('data-display', false);
		}, false);	
	
	});
}

function forEachQuerySelector (selector, fn) {
	Array.prototype.forEach.call(document.querySelectorAll(selector), fn); 
}

})();
