<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:sfm="http://www.utc.fr/ics/scenari/v3/filemeta" 
	xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive"
	xmlns:of="scpf.org:office" xmlns:wm="scpf.org:webMedia" xmlns:xalan="http://xml.apache.org/xalan" xmlns:redirect="http://xml.apache.org/xalan/redirect"
	extension-element-prefixes="redirect xalan redirect sfm">

	<xsl:import href="utils.xsl" />
	
	<xsl:output encoding="UTF-8" indent="yes" method="xml" />

	<xsl:strip-space elements="*" />
	
	<xsl:template match="of:richStream">
		<wm:segmentation>
			<xsl:apply-templates select="@* | node()" />
		</wm:segmentation>
	</xsl:template>
	
	<xsl:template match="of:richStreamM">
		<wm:segmentationM>
			<xsl:apply-templates select="@* | node()" />
		</wm:segmentationM>
	</xsl:template>
	
	<xsl:template match="of:richStream/sp:chapter">
		<sp:part>
			<xsl:apply-templates select="@* | node()" />
		</sp:part>
	</xsl:template>
	
	<xsl:template match="of:richStreamChapter">
		<wm:part>
			<xsl:apply-templates select="@* | node()" />
		</wm:part>
	</xsl:template>
	
	<xsl:template match="of:richStreamChapterM">
		<wm:partM>
			<xsl:apply-templates select="@* | node()" />
		</wm:partM>
	</xsl:template>
	
	<xsl:template match="of:richStreamM/sp:optsPub" />
	
	<xsl:template match="of:segment">
		<wm:segment>
			<xsl:apply-templates select="@* | node()" />
		</wm:segment>
	</xsl:template>
		
	<xsl:template match="of:segmentM">
		<xsl:variable name="vPrecSegmentEnd" select="number((ancestor::of:segment/preceding::of:illustration//sp:end)[last()])" />
		<xsl:variable name="vPrecParentEnd" select="if($vPrecSegmentEnd, $vPrecSegmentEnd, 0)" />
		
		<wm:segmentM>
			<xsl:apply-templates select="@* | sp:title" />
			<sp:timeCode>
				<wm:timeCode>
					<sp:begin>
						<xsl:value-of select="$vPrecParentEnd" />
					</sp:begin>
					<sp:end>
						<xsl:value-of select="(ancestor::of:segment//sp:end)[last()]" />
					</sp:end>
				</wm:timeCode>
			</sp:timeCode>
			<xsl:apply-templates select="@* | sp:id | sp:optsPub" />
		</wm:segmentM>
	</xsl:template>
	
	<xsl:template match="of:segOptsPubFlashM">
		<wm:segOptsPubFlashM>
			<xsl:apply-templates select="@* | node()" />
		</wm:segOptsPubFlashM>
	</xsl:template>
	
	<xsl:template match="sp:illustrTransition">
		<sp:timeRangeTransition>
			<xsl:apply-templates select="@* | node()" />
		</sp:timeRangeTransition>
	</xsl:template>
	
	<xsl:template match="sp:illustration">
		<sp:timeRange>
			<xsl:apply-templates select="@* | node()" />
		</sp:timeRange>
	</xsl:template>
	
	<xsl:template match="of:illustration">
		<wm:timeRange>
			<xsl:apply-templates select="@* | node()" />
		</wm:timeRange>
	</xsl:template>
	
	<xsl:template match="of:illustrationM">
		<wm:timeCode>
			<xsl:apply-templates select="@* | node()" />
		</wm:timeCode>
	</xsl:template>
	
	<xsl:template match="of:illustrationM/sp:end">
		<xsl:variable name="vPrecSegmentEnd" select="number((ancestor::of:segment/preceding::of:illustration//sp:end)[last()])" />
		<xsl:variable name="vPrecParentEnd" select="if($vPrecSegmentEnd, $vPrecSegmentEnd, 0)" />
		<xsl:variable name="vPrecIllustrEnd" select="number((preceding::of:illustration//sp:end)[last()])" />
		<xsl:variable name="vPrecEnd" select="if($vPrecIllustrEnd, $vPrecIllustrEnd, $vPrecParentEnd)" />
		<sp:begin>
			<xsl:value-of select="$vPrecEnd - $vPrecParentEnd" />
		</sp:begin>
		<sp:end>
			<xsl:value-of select="number(.) - $vPrecParentEnd" />
		</sp:end>
	</xsl:template>
	
	<xsl:template match="of:illustration/sp:content/of:sTxt | of:segment/sp:description/of:sTxt">
		<wm:illustrationText>
			<xsl:apply-templates select="@* | node()" />
		</wm:illustrationText>
	</xsl:template>
	
	<xsl:template match="sp:summary | sp:extras | sp:credits" />
	
	<xsl:template match="of:rss">
		<wm:rss>
			<xsl:apply-templates select="@* | node()" />
		</wm:rss>
	</xsl:template>
	
	<xsl:template match="of:rssM">
		<wm:rssM>
			<xsl:apply-templates select="@* | node()" />
		</wm:rssM>
	</xsl:template>
	
	<xsl:template match="of:rssItemM">
		<wm:rssItemM>
			<xsl:apply-templates select="@* | node()" />
		</wm:rssItemM>
	</xsl:template>
	
	<xsl:template match="of:rssCompItemM">
		<wm:rssCompItemM>
			<xsl:apply-templates select="@* | node()" />
		</wm:rssCompItemM>
	</xsl:template>
	
	<xsl:template match="sfm:flv/of:animM | sfm:mp3/of:animM">
		<wm:altMediaM>
			<xsl:apply-templates select="@* | node()" />
		</wm:altMediaM>
	</xsl:template>
	
	<xsl:template match="sp:webMedia">
		<xsl:copy>
			<xsl:attribute name="sc:refUri">
				<xsl:call-template name="tChangeExtension">
					<xsl:with-param name="pFilename" select="@sc:refUri" />
					<xsl:with-param name="pNewExtension">montage.xml</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:apply-templates select="node()" />
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="@* | node()">
		<xsl:copy>
			<xsl:apply-templates select="@* | node()" />
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
