<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:sc="http://www.utc.fr/ics/scenari/v3/core"
	xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive"
	xmlns:of="scpf.org:office" xmlns:op="utc.fr:ics/opale3" xmlns:wm="scpf.org:webMedia" xmlns:sfm="http://www.utc.fr/ics/scenari/v3/filemeta">

	<xsl:output encoding="UTF-8" indent="yes" method="xml" />

	<xsl:template match="sc:uLink[@role='url' and ancestor::op:bibTxt]/@role"/>

	<xsl:template match="op:bib">
		<of:doc>
			<xsl:if test="op:bibM">
				<of:docM>
					<xsl:if test="op:bibM/sp:id">
						<sp:title>
							<xsl:value-of select="op:bibM/sp:id"/>
						</sp:title>
					</xsl:if>
					<xsl:if test="op:bibM/sp:desc">
						<sp:ref>
							<of:docTxt>
								<xsl:apply-templates select="op:bibM/sp:desc/op:bibTxt/*" />
							</of:docTxt>
						</sp:ref>
					</xsl:if>
					<sp:src/>
				</of:docM>
			</xsl:if>	
		</of:doc>
	</xsl:template>

	<xsl:template match="op:glos">
		<of:def>
			<xsl:if test="op:glosM/sp:term">
				<of:defM>
					<xsl:copy-of select="op:glosM/sp:term"/>
				</of:defM>
			</xsl:if>
			<xsl:if test="op:glosM/sp:def">
				<xsl:apply-templates select="op:glosM/sp:def" />
			</xsl:if>				
		</of:def>
	</xsl:template>

	<xsl:template match="op:ref">
		<wm:index>
			<xsl:if test="op:refM/sp:ref">
				<wm:indexM>
					<sp:term>
						<xsl:value-of select="op:refM/sp:ref"/>
					</sp:term>
				</wm:indexM>
			</xsl:if>
			<xsl:if test="op:refM/sp:desc">
				<sp:def>
					<xsl:apply-templates select="op:refM/sp:desc/*" />
				</sp:def>
			</xsl:if>				
		</wm:index>
	</xsl:template>

	<xsl:template match="sc:uLink[not(contains(@sc:refUri,'id:'))]">
		<xsl:choose>
			<xsl:when test="contains(@sc:refUri,'.ref')">
				<sc:uLink role="{@role}" sc:refUri="{concat(@sc:refUri,'s')}">
					<xsl:value-of select="."/>
				</sc:uLink>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates select="node() | @*" />
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>		
	</xsl:template>

	<xsl:template match="sp:tag[not(contains(@sc:refUri,'id:'))]">
		<xsl:choose>
			<xsl:when test="contains(@sc:refUri,'.ref')">
				<sp:tag sc:refUri="{concat(@sc:refUri,'s')}"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates select="node() | @*" />
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>		
	</xsl:template>

	<xsl:template match="sfm:mp4">
		<sfm:mp4_f4v>
  			<xsl:apply-templates select="./*" />
 		</sfm:mp4_f4v>
	</xsl:template>

	<xsl:template match="wm:segmentationM/sp:commonMeta"/>

	<xsl:template match="of:commonM">
		<wm:commonM>
			<xsl:apply-templates select="./*" />
		</wm:commonM>		
	</xsl:template>

	<xsl:template match="node() | @*">
		<xsl:copy>
			<xsl:apply-templates select="node() | @*" />
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
