<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:sc="http://www.utc.fr/ics/scenari/v3/core" xmlns:sfm="http://www.utc.fr/ics/scenari/v3/filemeta" 
	xmlns:sp="http://www.utc.fr/ics/scenari/v3/primitive" xmlns:wr="scpf.org:webRadio"
	xmlns:of="scpf.org:office" xmlns:xalan="http://xml.apache.org/xalan" xmlns:redirect="http://xml.apache.org/xalan/redirect"
	extension-element-prefixes="redirect xalan redirect"
	exclude-result-prefixes="wr">

	<xsl:include href="utils.xsl" />

	<xsl:param name="pWspPath" />

	<xsl:output encoding="UTF-8" indent="yes" method="xml" />

	<xsl:strip-space elements="*" />

	<xsl:template match="wr:webRadio">
		<of:richStream>
			<xsl:apply-templates select="wr:wRadioMeta" />
			<xsl:apply-templates select="wr:wRadioMeta" mode="abstract" />
			<xsl:apply-templates select="sp:segment" />
			<xsl:apply-templates select="wr:wRadioMeta" mode="documents" />
			<xsl:apply-templates select="wr:wRadioMeta" mode="credits" />
		</of:richStream>
	</xsl:template>

	<xsl:template match="wr:wRadioMeta">
		<of:richStreamM>
			<xsl:copy-of select="sp:title" />
			<sp:stream sc:refUri="{../sp:segment/@sc:refUri}" />
		</of:richStreamM>
	</xsl:template>

	<xsl:template match="wr:wRadioMeta" mode="abstract">
		<xsl:apply-templates select="sp:abstract" />
	</xsl:template>

	<xsl:template match="sp:abstract">
		<sp:summary>
			<xsl:apply-templates>
				<xsl:with-param name="pModel">sTxt</xsl:with-param>
			</xsl:apply-templates>
		</sp:summary>
	</xsl:template>

	<xsl:template match="wr:wRadioMeta" mode="credits">
		<xsl:if test="sp:date">
			<sp:credits>
				<of:section>
					<of:sectionM>
						<sp:title>Crédits</sp:title>
					</of:sectionM>
					<sp:content>
						<of:fragment>
							<sp:info>
								<of:block>
									<of:blockM />
									<sp:co>
										<of:flow>
											<sp:txt>
												<of:txt>
													<xsl:apply-templates
														select="sp:date | sp:author | sp:prod | sp:realisation | sp:assistantprod" />
													<xsl:if test="sp:invite">
													  <sc:para>
													  	<sc:inlineStyle role="emphasis">Invités : </sc:inlineStyle>
													  <xsl:apply-templates select="sp:invite" />
													  </sc:para>
													</xsl:if>
												</of:txt>
											</sp:txt>
										</of:flow>
									</sp:co>
								</of:block>
							</sp:info>
						</of:fragment>
					</sp:content>
				</of:section>
			</sp:credits>
		</xsl:if>
	</xsl:template>


	<xsl:template match="sp:date">
		<sc:para>
			<sc:inlineStyle role="emphasis">Date de diffusion : </sc:inlineStyle>
			<xsl:apply-templates />
		</sc:para>
	</xsl:template>

	<xsl:template match="sp:author">
		<sc:para>
			<sc:inlineStyle role="emphasis">Auteur : </sc:inlineStyle>
			<xsl:apply-templates />
		</sc:para>
	</xsl:template>

	<xsl:template match="sp:prod">
		<sc:para>
			<sc:inlineStyle role="emphasis">Production : </sc:inlineStyle>
			<xsl:apply-templates />
		</sc:para>
	</xsl:template>

	<xsl:template match="sp:realisation">
		<sc:para>
			<sc:inlineStyle role="emphasis">Réalisation : </sc:inlineStyle>
			<xsl:apply-templates />
		</sc:para>
	</xsl:template>

	<xsl:template match="sp:assistantprod">
		<sc:para>
			<sc:inlineStyle role="emphasis">Assistant de production : </sc:inlineStyle>
			<xsl:apply-templates />
		</sc:para>
	</xsl:template>

	<xsl:template match="sp:invite">
		<xsl:apply-templates />
		<xsl:if test="following-sibling::sp:invite">
			<xsl:text>, </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="wr:wRadioMeta" mode="documents">
		<xsl:if test="sp:documents">
			<sp:extras>
				<of:section>
					<of:sectionM>
						<sp:title>Documents</sp:title>
					</of:sectionM>
					<sp:content>
						<of:fragment>
							<sp:info>
								<of:block>
									<of:blockM />
									<sp:co>
										<of:flow>
											<sp:txt>
												<of:txt>
													<xsl:apply-templates select="sp:documents" />
												</of:txt>
											</sp:txt>
										</of:flow>
									</sp:co>
								</of:block>
							</sp:info>
						</of:fragment>
					</sp:content>
				</of:section>
			</sp:extras>
		</xsl:if>
	</xsl:template>

	<xsl:template match="sp:documents">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="sp:segment">
		<xsl:variable name="vIllustDuration"
			select="(sc:effects/@end - sc:effects/@start) div if(wr:segMeta/sp:illustr,count(wr:segMeta/sp:illustr), 1)" />
		<xsl:apply-templates select="wr:segMeta">
			<xsl:with-param name="pIllustrDuration" select="$vIllustDuration" />
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="wr:segMeta">
		<xsl:param name="pIllustrDuration" />
		<sp:segment>
			<of:segment>
				<of:segmentM>
					<xsl:copy-of select="sp:title" />
				</of:segmentM>
				<xsl:apply-templates select="sp:desc" />
				
				<xsl:choose>
					<xsl:when test="sp:illustr">
						<xsl:apply-templates select="sp:illustr">
							<xsl:with-param name="pIllustrDuration" select="$pIllustrDuration" />
						</xsl:apply-templates>
					</xsl:when>
					<xsl:otherwise>
						<sp:illustration>
							<of:illustration>
								<of:illustrationM>
									<sp:end>
										<xsl:value-of
											select="format-number(ancestor::sp:segment[1]/sc:effects/@start + $pIllustrDuration, '#.000')"
										 />
									</sp:end>
								</of:illustrationM>
							</of:illustration>
						</sp:illustration>
					</xsl:otherwise>
				</xsl:choose>

				<xsl:apply-templates select="sp:comp" />
			</of:segment>
		</sp:segment>
	</xsl:template>

	<xsl:template match="sp:desc">
		<sp:description>
			<xsl:apply-templates />
		</sp:description>
	</xsl:template>

	<xsl:template match="sp:illustr">
		<xsl:param name="pIllustrDuration" />
		<xsl:variable name="vEnd"
			select="ancestor::sp:segment[1]/sc:effects/@start + ($pIllustrDuration * (count(preceding-sibling::sp:illustr) + 1))" />
		<sp:illustration>
			<xsl:apply-templates>
				<xsl:with-param name="pEnd" select="$vEnd" />
			</xsl:apply-templates>
		</sp:illustration>
	</xsl:template>

	<xsl:template match="wr:illustr">
		<xsl:param name="pEnd" />
		<of:illustration>
			<of:illustrationM>
				<sp:end>
					<xsl:value-of select="format-number($pEnd,'#.000')" />
				</sp:end>
			</of:illustrationM>
			<xsl:apply-templates />
		</of:illustration>
	</xsl:template>

	<xsl:template match="wr:illustr/sp:img  | wr:illustr/sp:anim | wr:illustr/sp:video">
		<sp:content>
			<xsl:copy-of select="@sc:refUri" />
		</sp:content>
		<xsl:if test="wr:simpleTitle">
			<xsl:call-template name="tWriteMeta">
				<xsl:with-param name="pBinaryMeta">imgM</xsl:with-param>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template match="wr:illustr/sp:text">
		<sp:content>
			<xsl:apply-templates>
				<xsl:with-param name="pModel">sTxt</xsl:with-param>
			</xsl:apply-templates>
		</sp:content>
	</xsl:template>

	<xsl:template match="sp:comp">
		<sp:complement>
			<xsl:attribute name="sc:refUri">
				<xsl:call-template name="tChangeExtension">
					<xsl:with-param name="pFilename" select="@sc:refUri" />
					<xsl:with-param name="pNewExtension">section</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
		</sp:complement>
	</xsl:template>

	<xsl:template match="wr:comp | wr:compSect">
		<xsl:variable name="vMetaContent">
			<xsl:apply-templates select="wr:compMeta" mode="content" />
		</xsl:variable>
		
		<of:section>
			<xsl:apply-templates select="wr:compMeta | wr:simpleTitle" />

			<sp:content>
				<of:fragment>
					<xsl:if test="string-length($vMetaContent) or *[not(self::wr:compMeta | self::wr:simpleTitle | self::sp:section)]">
						<sp:info>
							<of:block>
								<of:blockM />
								<sp:co>
									<of:flow>
										<sp:txt>
											<of:txt>
												<xsl:copy-of select="$vMetaContent" />
												<xsl:apply-templates select="*[not(self::wr:compMeta | self::wr:simpleTitle |  self::sp:section)]">
													<xsl:with-param name="pModel" select="false()" />
												</xsl:apply-templates>
											</of:txt>
										</sp:txt>
									</of:flow>
								</sp:co>
							</of:block>
						</sp:info>
					</xsl:if>
				</of:fragment>
			</sp:content>
			<xsl:apply-templates select="sp:section" />
		</of:section>
	</xsl:template>
	
	<xsl:template match="sp:section">
		<sp:sec>
			<xsl:apply-templates />
		</sp:sec>
	</xsl:template>

	<xsl:template match="wr:compMeta">
		<of:sectionM>
			<xsl:copy-of select="sp:title" />
		</of:sectionM>
	</xsl:template>
	
	<xsl:template match="wr:compMeta[sp:author | sp:date]" mode="content">
		<sc:para><sc:inlineStyle role="emphasis">Auteur : </sc:inlineStyle><xsl:value-of select="sp:author"/></sc:para>
		<sc:para><sc:inlineStyle role="emphasis">Date : </sc:inlineStyle><xsl:value-of select="sp:date"/></sc:para>
	</xsl:template>
	
	<xsl:template match="wr:compMeta" mode="content" />
	
	<xsl:template match="wr:simpleTitle">
		<of:sectionM>
			<sp:title>
				<xsl:value-of select="sc:fullTitle" />
			</sp:title>
		</of:sectionM>
	</xsl:template>


	<xsl:template match="sp:document">
		<sc:para>
			<sc:uLink role="ref" sc:refUri="{sp:docRes/@sc:refUri}">
				<xsl:value-of select="sp:docTitle" />
			</sc:uLink>
		</sc:para>
	</xsl:template>
	
	<xsl:template match="sp:url">
		<sc:para>
			<sc:uLink role="url" url="{wr:url/sp:urlText}">
				<xsl:value-of select="wr:url/sp:urlTitle" />
			</sc:uLink>
		</sc:para>
	</xsl:template>
	
	
	<xsl:template match="sp:img">
		<sc:extBlock role="img">
			<xsl:copy-of select="@sc:refUri" />
		</sc:extBlock>
		<xsl:if test="wr:simpleTitle">
			<xsl:call-template name="tWriteMeta">
				<xsl:with-param name="pBinaryMeta">imgM</xsl:with-param>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="sp:mp3 | sp:video | sp:interact">
		<sc:extBlock role="anim">
			<xsl:copy-of select="@sc:refUri" />
		</sc:extBlock>
		<xsl:if test="wr:simpleTitle">
			<xsl:call-template name="tWriteMeta" />
		</xsl:if>
	</xsl:template>

	<xsl:template name="tWriteMeta">
		<xsl:param name="pBinaryMeta">animM</xsl:param>
		
		<xsl:variable name="vExtension">
			<xsl:call-template name="tToLower">
				<xsl:with-param name="pString">
					<xsl:call-template name="tExtension">
						<xsl:with-param name="pFilename" select="@sc:refUri" />
					</xsl:call-template>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="vBinaryCode">
			<xsl:choose>
				<xsl:when test="$vExtension = 'jpeg' or $vExtension = 'jpg'">jpg_jpeg</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$vExtension"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<redirect:write select="concat($pWspPath, @sc:refUri, '/meta.xml')">
			<xsl:element name="sfm:{$vBinaryCode}">
				<xsl:element name="of:{$pBinaryMeta}">
					<sp:title>
						<xsl:value-of select="wr:simpleTitle/sc:fullTitle" />
					</sp:title>
				</xsl:element>
			</xsl:element>
		</redirect:write>
	</xsl:template>

	<xsl:template match="sp:pdf">
		<sc:para>
			<sc:uLink role="ref">
				<xsl:copy-of select="@sc:refUri"/>
				<xsl:choose>
					<xsl:when test="wr:simpleTitle">
						<xsl:value-of select="wr:simpleTitle/sc:fullTitle"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="tBasename">
							<xsl:with-param name="pFilename" select="@sc:refUri" />
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</sc:uLink>
		</sc:para>
	</xsl:template>
	
	<xsl:template match="sp:text">
		<xsl:param name="pModel" />
		<xsl:apply-templates>
			<xsl:with-param name="pModel" select="$pModel" />
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="wr:simpleText">
		<xsl:param name="pModel">sTxt</xsl:param>

		<xsl:choose>
			<xsl:when test="$pModel">
				<xsl:element name="of:{$pModel}">
					<xsl:apply-templates />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="wr:text">
		<xsl:param name="pModel">txt</xsl:param>

		<xsl:choose>
			<xsl:when test="$pModel">
				<xsl:element name="of:{$pModel}">
					<xsl:apply-templates />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="sc:simpleList">
		<sc:itemizedList>
			<xsl:apply-templates />
		</sc:itemizedList>
	</xsl:template>
	
	<xsl:template match="sc:member">
		<sc:listItem>
			<sc:para>
				<xsl:apply-templates />
			</sc:para>
		</sc:listItem>
	</xsl:template>

	<xsl:template match="sc:uLink">
		<sc:uLink role="url">
			<xsl:copy-of select="@url" />
			<xsl:apply-templates />
		</sc:uLink>
	</xsl:template>
	
	<!-- RSS -->
	<xsl:template match="wr:rss | wr:rssMeta | wr:rssItemMeta | wr:rssCompItemMeta">
		<xsl:element name="of:{local-name(.)}">
			<xsl:apply-templates select="@* | node()" />
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="wr:rss//sp:webRadio">
		<sp:webMedia>
			<xsl:attribute name="sc:refUri">
				<xsl:call-template name="tChangeExtension">
					<xsl:with-param name="pFilename" select="@sc:refUri" />
					<xsl:with-param name="pNewExtension">richmedia</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:apply-templates select="node()" />
		</sp:webMedia>
	</xsl:template>
	
	<xsl:template match="wr:rss/sp:comp">
		<sp:complement>
			<xsl:attribute name="sc:refUri">
				<xsl:call-template name="tChangeExtension">
					<xsl:with-param name="pFilename" select="@sc:refUri" />
					<xsl:with-param name="pNewExtension">section</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:apply-templates select="node()" />
		</sp:complement>
	</xsl:template>

	<xsl:template match="@* | node()">
		<xsl:copy>
			<xsl:apply-templates select="@* | node()" />
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
